package Catalyst::Plugin::AccessLog::BOKUTIN;
{
  $Catalyst::Plugin::AccessLog::BOKUTIN::VERSION = '0.132596';
} 

use Moose::Role;
with qw(Catalyst::Plugin::AccessLog);
use namespace::autoclean;

use Sys::Syslog qw(openlog syslog);
use Time::HiRes qw(gettimeofday tv_interval);

has additional_access_log_field_stash_key => (
    is      => "rw",
    isa     => "Str",
    default => sub { "additional_access_log_field" }
);

before setup_finalize => sub {
    my $c = shift;

    my $default_config = {
        enable_stats => 0,
        formatter => {
            class => 'Catalyst::Plugin::AccessLog::BOKUTIN::Formatter',
        },
        target => sub {
            my ($message, $c) = @_;

            openlog($c->config->{name}, "pid", "local2");
            syslog("info", $message);
        },
    };
    $c->config->{'Plugin::AccessLog'} = Catalyst::Utils::merge_hashes(
        $default_config,
        $c->config->{'Plugin::AccessLog'},
    );
};

around dispatch => sub {
    my $orig = shift;
    my $c = shift;

    my $start   = [gettimeofday];
    my $ret     = $c->$orig(@_);
    my $end     = [gettimeofday];
    my $elapsed = tv_interval($start, $end);

    $c->stash->{__elapsed} = $elapsed;

    $ret;
};

1;

__END__

=pod

=head1 NAME

Catalyst::Plugin::AccessLog::BOKUTIN

=head1 VERSION

version 0.132596

=head1 AUTHOR

Tomohiro Hosaka <bokutin@bokut.in>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by Tomohiro Hosaka.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
