package Catalyst::Plugin::AccessLog::BOKUTIN::Formatter;
{
  $Catalyst::Plugin::AccessLog::BOKUTIN::Formatter::VERSION = '0.132596';
}

use feature ":5.10";
use Moose;
extends qw(Catalyst::Plugin::AccessLog::Formatter);

use JSON::XS ();
use POSIX ();
use Params::Util qw(_HASH);

override format_line => sub {
    my ($self, $c) = @_;

    # LogFormat "%v %h %l %u %t "%r" %>s %b "%{Referer}i" "%{User-Agent}i"" combined
    # format /^(?<vhost>[^ ]*) (?<host>[^ ]*) [^ ]* (?<user>[^ ]*) \[(?<time>[^\]]*)\] "(?<method>\S+)(?: +(?<path>[^ ]*) +\S*)?" (?<code>[^ ]*) (?<size>[^ ]*)(?: "(?<referer>[^\"]*)" "(?<agent>[^\"]*)")?$/

    my %col;

    # Apache互換
    #$col{vhost}  = $self->get_item($c, 'v');
    #$col{vhost}  = $c->req->base->host_port;
    $col{vhost}  = $c->req->base->host;
    $col{host}   = $self->get_item($c, 'h');
    $col{user}   = $self->get_item($c, 'u');
    $col{time}   = $self->get_item($c, 't');
    $col{method} = $c->req->method;
    $col{path}   = do {
        my $path  = $c->req->uri->path;
        my $query = $c->req->uri->query;
        if (defined $query && length $query) {
            $query = "?$query";
        } else {
            $query = "";
        }
        "${path}${query}";
    };
    $col{code}    = $self->get_item($c, 's');
    $col{size}    = $self->get_item($c, 'b');
    $col{referer} = $c->req->referer;
    $col{agent}   = $c->req->user_agent;

    # その他
    $col{ident}     = $c->config->{name};
    $col{time}      = POSIX::strftime("%Y-%m-%d %H:%M:%S", localtime(time));
    $col{userid}    = $self->_userid($c);
    $col{username}  = $self->_username($c);
    $col{sessionid} = $self->get_item($c, 'sessionid');
    $col{elapsed}   = $c->stash->{__elapsed};

    if (my $hash = _HASH($c->stash->{ $c->additional_access_log_field_stash_key })) {
        $col{$_} = $hash->{$_} for keys %$hash;
    }

    state $json = JSON::XS->new->canonical;
    return $json->encode(\%col);
};

sub _userid {
    my ($self, $c) = @_;
    return "-" unless $c->can('user') and $c->user_exists;
    $c->user->id // "-";
}

sub _username {
    my ($self, $c) = @_;
    return "-" unless $c->can('user') and $c->user_exists;
    $c->user->get('username') // "-";
}

__PACKAGE__->meta->make_immutable;
no Moose;
1;

__END__

=pod

=head1 NAME

Catalyst::Plugin::AccessLog::BOKUTIN::Formatter

=head1 VERSION

version 0.132596

=head1 AUTHOR

Tomohiro Hosaka <bokutin@bokut.in>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by Tomohiro Hosaka.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
